@@ -78,7 +78,7 @@ pathlib = _LazyModule.load('pathlib', 'pathlib')
 platform = _LazyModule.load('platform', 'platform')
 functools = _LazyModule.load('functools', 'functools')
 subprocess = _LazyModule.load('subprocess', 'subprocess')
-__version__ = "0.2.4"
+__version__ = "0.2.5"
 
 
 @functools.lru_cache(1)
@@ -437,8 +437,8 @@ def bash_completions(
     commprefix = os.path.commonprefix(list(out))
     strip_len = 0
     strip_prefix = prefix.strip("\"'")
-    while strip_len < len(prefix):
-        if commprefix.startswith(strip_prefix[strip_len:]):
+    while strip_len < len(strip_prefix) and strip_len < len(commprefix):
+        if commprefix[strip_len] == strip_prefix[strip_len]:
             break
         strip_len += 1
 
@@ -528,7 +528,7 @@ def complete_completer(prefix, line, start, end, ctx):
     if len(args) == 0 or args[0] != "completer":
         return None
     curix = args.index(prefix)
-    compnames = set(builtins.__xonsh_completers__.keys())
+    compnames = set(builtins.__xonsh__.completers.keys())
     if curix == 1:
         possible = {"list", "help", "add", "remove"}
     elif curix == 2:
@@ -542,7 +542,7 @@ def complete_completer(prefix, line, start, end, ctx):
         if args[1] != "add":
             raise StopIteration
         if curix == 3:
-            possible = {i for i, j in builtins.__xonsh_ctx__.items() if callable(j)}
+            possible = {i for i, j in builtins.__xonsh__.ctx.items() if callable(j)}
         elif curix == 4:
             possible = (
                 {"start", "end"}
@@ -629,7 +629,7 @@ def get_filter_function():
     Return an appropriate filtering function for completions, given the valid
     of $CASE_SENSITIVE_COMPLETIONS
     """
-    csc = builtins.__xonsh_env__.get("CASE_SENSITIVE_COMPLETIONS")
+    csc = builtins.__xonsh__.env.get("CASE_SENSITIVE_COMPLETIONS")
     if csc:
         return _filter_normal
     else:
@@ -711,16 +711,16 @@ def _add_one_completer(name, func, loc="end"):
     new = collections.OrderedDict()
     if loc == "start":
         new[name] = func
-        for (k, v) in builtins.__xonsh_completers__.items():
+        for (k, v) in builtins.__xonsh__.completers.items():
             new[k] = v
     elif loc == "end":
-        for (k, v) in builtins.__xonsh_completers__.items():
+        for (k, v) in builtins.__xonsh__.completers.items():
             new[k] = v
         new[name] = func
     else:
         direction, rel = loc[0], loc[1:]
         found = False
-        for (k, v) in builtins.__xonsh_completers__.items():
+        for (k, v) in builtins.__xonsh__.completers.items():
             if rel == k and direction == "<":
                 new[name] = func
                 found = True
@@ -730,13 +730,13 @@ def _add_one_completer(name, func, loc="end"):
                 found = True
         if not found:
             new[name] = func
-    builtins.__xonsh_completers__.clear()
-    builtins.__xonsh_completers__.update(new)
+    builtins.__xonsh__.completers.clear()
+    builtins.__xonsh__.completers.update(new)
 
 
 def _list_completers(args, stdin=None, stack=None):
     o = "Registered Completer Functions: \n"
-    _comp = builtins.__xonsh_completers__
+    _comp = builtins.__xonsh__.completers
     ml = max((len(i) for i in _comp), default=0)
     _strs = []
     for c in _comp:
@@ -755,10 +755,10 @@ def _remove_completer(args, stdin=None, stack=None):
         err = "completer remove takes exactly 1 argument."
     else:
         name = args[0]
-        if name not in builtins.__xonsh_completers__:
+        if name not in builtins.__xonsh__.completers:
             err = ("The name %s is not a registered " "completer function.") % name
     if err is None:
-        del builtins.__xonsh_completers__[name]
+        del builtins.__xonsh__.completers[name]
         return
     else:
         return None, err + "\n", 1
@@ -774,11 +774,11 @@ def _register_completer(args, stdin=None, stack=None):
     else:
         name = args[0]
         func_name = args[1]
-        if name in builtins.__xonsh_completers__:
+        if name in builtins.__xonsh__.completers:
             err = ("The name %s is already a registered " "completer function.") % name
         else:
-            if func_name in builtins.__xonsh_ctx__:
-                func = builtins.__xonsh_ctx__[func_name]
+            if func_name in builtins.__xonsh__.ctx:
+                func = builtins.__xonsh__.ctx[func_name]
                 if not callable(func):
                     err = "%s is not callable" % func_name
             else:
@@ -904,7 +904,7 @@ def complete_command(cmd, line, start, end, ctx):
     space = " "
     out = {
         s + space
-        for s in builtins.__xonsh_commands_cache__
+        for s in builtins.__xonsh__.commands_cache
         if get_filter_function()(s, cmd)
     }
     if xp.ON_WINDOWS:
@@ -938,7 +938,7 @@ def complete_skipper(cmd, line, start, end, ctx):
     if len(parts) == skip_part_num + 1:
         comp_func = complete_command
     else:
-        comp = builtins.__xonsh_shell__.shell.completer
+        comp = builtins.__xonsh__.shell.shell.completer
         comp_func = comp.complete
 
     skip_len = len(" ".join(line[:skip_part_num])) + 1
@@ -977,7 +977,7 @@ def complete_from_man(prefix, line, start, end, ctx):
     """
     global OPTIONS, OPTIONS_PATH
     if OPTIONS is None:
-        datadir = builtins.__xonsh_env__["XONSH_DATA_DIR"]
+        datadir = builtins.__xonsh__.env["XONSH_DATA_DIR"]
         OPTIONS_PATH = os.path.join(datadir, "man_completions_cache")
         try:
             with open(OPTIONS_PATH, "rb") as f:
@@ -1027,7 +1027,7 @@ def PATTERN_NEED_QUOTES():
 
 def cd_in_command(line):
     """Returns True if "cd" is a token in the line, False otherwise."""
-    lexer = builtins.__xonsh_execer__.parser.lexer
+    lexer = builtins.__xonsh__.execer.parser.lexer
     lexer.reset()
     lexer.input(line)
     have_cd = False
@@ -1065,7 +1065,7 @@ def _path_from_partial_string(inp, pos=None):
     except (SyntaxError, ValueError):
         return None
     if isinstance(val, bytes):
-        env = builtins.__xonsh_env__
+        env = builtins.__xonsh__.env
         val = val.decode(
             encoding=env.get("XONSH_ENCODING"), errors=env.get("XONSH_ENCODING_ERRORS")
         )
@@ -1087,7 +1087,7 @@ def _normpath(p):
         p = os.path.join(os.curdir, p)
     if trailing_slash:
         p = os.path.join(p, "")
-    if xp.ON_WINDOWS and builtins.__xonsh_env__.get("FORCE_POSIX_PATHS"):
+    if xp.ON_WINDOWS and builtins.__xonsh__.env.get("FORCE_POSIX_PATHS"):
         p = p.replace(os.sep, os.altsep)
     return p
 
@@ -1106,7 +1106,7 @@ def _env(prefix):
     if prefix.startswith("$"):
         key = prefix[1:]
         return {
-            "$" + k for k in builtins.__xonsh_env__ if get_filter_function()(k, key)
+            "$" + k for k in builtins.__xonsh__.env if get_filter_function()(k, key)
         }
     return ()
 
@@ -1125,7 +1125,7 @@ def _dots(prefix):
 
 def _add_cdpaths(paths, prefix):
     """Completes current prefix using CDPATH"""
-    env = builtins.__xonsh_env__
+    env = builtins.__xonsh__.env
     csc = env.get("CASE_SENSITIVE_COMPLETIONS")
     glob_sorted = env.get("GLOB_SORTED")
     for cdp in env.get("CDPATH"):
@@ -1147,7 +1147,7 @@ def _quote_to_use(x):
 
 
 def _quote_paths(paths, start, end, append_end=True):
-    expand_path = builtins.__xonsh_expand_path__
+    expand_path = builtins.__xonsh__.expand_path
     out = set()
     space = " "
     backslash = "\\"
@@ -1255,7 +1255,7 @@ def _subsequence_match_iter(ref, typed):
 
 def _expand_one(sofar, nextone, csc):
     out = set()
-    glob_sorted = builtins.__xonsh_env__.get("GLOB_SORTED")
+    glob_sorted = builtins.__xonsh__.env.get("GLOB_SORTED")
     for i in sofar:
         _glob = os.path.join(_joinpath(i), "*") if i is not None else "*"
         for j in xt.iglobpath(_glob, sort_result=glob_sorted):
@@ -1282,7 +1282,7 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
             append_end = False
     tilde = "~"
     paths = set()
-    env = builtins.__xonsh_env__
+    env = builtins.__xonsh__.env
     csc = env.get("CASE_SENSITIVE_COMPLETIONS")
     glob_sorted = env.get("GLOB_SORTED")
     prefix = glob.escape(prefix)
@@ -1481,7 +1481,7 @@ def _complete_python(prefix, line, start, end, ctx):
     """
     if line != "":
         first = line.split()[0]
-        if first in builtins.__xonsh_commands_cache__ and first not in ctx:
+        if first in builtins.__xonsh__.commands_cache and first not in ctx:
             return set()
     filt = get_filter_function()
     rtn = set()
@@ -1520,7 +1520,7 @@ def _safe_eval(expr, ctx):
     a (None, None) tuple.
     """
     _ctx = None
-    xonsh_safe_eval = builtins.__xonsh_execer__.eval
+    xonsh_safe_eval = builtins.__xonsh__.execer.eval
     try:
         val = xonsh_safe_eval(expr, ctx, ctx, transform=False)
         _ctx = ctx
@@ -1558,7 +1558,7 @@ def attr_complete(prefix, ctx, filter_func):
         if _val_ is None and _ctx_ is None:
             continue
         a = getattr(val, opt)
-        if builtins.__xonsh_env__["COMPLETIONS_BRACKETS"]:
+        if builtins.__xonsh__.env["COMPLETIONS_BRACKETS"]:
             if callable(a):
                 rpl = opt + "("
             elif isinstance(a, (cabc.Sequence, cabc.Mapping)):
@@ -1660,8 +1660,8 @@ def complete_base(prefix, line, start, end, ctx):
 # amalgamated xonsh.completers.bash_completion
 def complete_from_bash(prefix, line, begidx, endidx, ctx):
     """Completes based on results from BASH completion."""
-    env = builtins.__xonsh_env__.detype()
-    paths = builtins.__xonsh_env__.get("BASH_COMPLETIONS", ())
+    env = builtins.__xonsh__.env.detype()
+    paths = builtins.__xonsh__.env.get("BASH_COMPLETIONS", ())
     command = xp.bash_command()
     return bash_completions(
         prefix,
